@@ -48,14 +48,13 @@ struct _GcalNewCalendarPage
   GtkEntry           *calendar_address_entry;
   EntryState          calendar_address_entry_state;
   GcalFileChooserButton *calendar_file_chooser_button;
-  GtkFileFilter      *calendar_file_filter;
   GtkWidget          *cancel_button;
   GtkWidget          *credentials_cancel_button;
   GtkWidget          *credentials_connect_button;
   GtkWindow          *credentials_dialog;
   GtkEntry           *credentials_password_entry;
   GtkEntry           *credentials_user_entry;
-  GtkColorChooser    *local_calendar_color_button;
+  GtkColorDialogButton *local_calendar_color_button;
   GtkEntry           *local_calendar_name_entry;
   GtkWidget          *web_sources_listbox;
   GtkWidget          *web_sources_revealer;
@@ -161,11 +160,11 @@ update_local_source (GcalNewCalendarPage *self)
     {
       g_autofree gchar *color_string = NULL;
       ESourceExtension *ext;
+      const GdkRGBA *color;
       ESource *source;
-      GdkRGBA color;
 
-      gtk_color_chooser_get_rgba (self->local_calendar_color_button, &color);
-      color_string = gdk_rgba_to_string (&color);
+      color = gtk_color_dialog_button_get_rgba (self->local_calendar_color_button);
+      color_string = gdk_rgba_to_string (color);
 
       /* Create the new source and add the needed extensions */
       source = e_source_new (NULL, NULL, NULL);
@@ -483,9 +482,9 @@ on_local_calendar_name_entry_text_changed_cb (GtkEntry            *entry,
 }
 
 static void
-on_local_calendar_color_button_color_changed_cb (GtkColorChooser     *chooser,
-                                                 GParamSpec          *pspec,
-                                                 GcalNewCalendarPage *self)
+on_local_calendar_color_button_rgba_changed_cb (GtkColorChooser     *chooser,
+                                                GParamSpec          *pspec,
+                                                GcalNewCalendarPage *self)
 {
   update_local_source (self);
 }
@@ -659,7 +658,6 @@ gcal_new_calendar_page_class_init (GcalNewCalendarPageClass *klass)
   gtk_widget_class_bind_template_child (widget_class, GcalNewCalendarPage, add_button);
   gtk_widget_class_bind_template_child (widget_class, GcalNewCalendarPage, calendar_address_entry);
   gtk_widget_class_bind_template_child (widget_class, GcalNewCalendarPage, calendar_file_chooser_button);
-  gtk_widget_class_bind_template_child (widget_class, GcalNewCalendarPage, calendar_file_filter);
   gtk_widget_class_bind_template_child (widget_class, GcalNewCalendarPage, cancel_button);
   gtk_widget_class_bind_template_child (widget_class, GcalNewCalendarPage, credentials_cancel_button);
   gtk_widget_class_bind_template_child (widget_class, GcalNewCalendarPage, credentials_connect_button);
@@ -678,7 +676,7 @@ gcal_new_calendar_page_class_init (GcalNewCalendarPageClass *klass)
   gtk_widget_class_bind_template_callback (widget_class, on_credential_entry_activate_cb);
   gtk_widget_class_bind_template_callback (widget_class, on_file_chooser_button_file_changed_cb);
   gtk_widget_class_bind_template_callback (widget_class, on_local_calendar_name_entry_text_changed_cb);
-  gtk_widget_class_bind_template_callback (widget_class, on_local_calendar_color_button_color_changed_cb);
+  gtk_widget_class_bind_template_callback (widget_class, on_local_calendar_color_button_rgba_changed_cb);
   gtk_widget_class_bind_template_callback (widget_class, on_url_entry_text_changed_cb);
   gtk_widget_class_bind_template_callback (widget_class, on_web_description_label_link_activated_cb);
 }
@@ -686,14 +684,7 @@ gcal_new_calendar_page_class_init (GcalNewCalendarPageClass *klass)
 static void
 gcal_new_calendar_page_init (GcalNewCalendarPage *self)
 {
-  GtkFileChooser *chooser;
-
   self->cancellable = g_cancellable_new ();
 
   gtk_widget_init_template (GTK_WIDGET (self));
-
-  gtk_file_filter_set_name (self->calendar_file_filter, _("Calendar files"));
-
-  chooser = gcal_file_chooser_button_get_filechooser (self->calendar_file_chooser_button);
-  gtk_file_chooser_add_filter (chooser, self->calendar_file_filter);
 }
