@@ -23,14 +23,11 @@
 ".color-%1$d {"\
 "  background-color: %2$s;"\
 "}"\
-".color-%1$d.horizontal.timed {"\
-"  background-color: transparent;"\
+".color-%1$d.timed {"\
+"  background-color: alpha(%2$s, 0.17);"\
 "}"\
-".color-%1$d.horizontal.timed box.icon-box{"\
+".color-%1$d.timed widget.edge{"\
 "  background-color: %2$s;"\
-"  border-radius: 2px;"\
-"  min-height: 16px;"\
-"  min-width: 16px;"\
 "}"\
 ".color-%1$d.slanted {"\
 "  background-color: transparent;"\
@@ -68,11 +65,6 @@
 "  background-repeat: no-repeat;"\
 "  background-origin: padding-box, padding-box, border-box, border-box;"\
 "  background-clip: padding-box, padding-box, border-box, border-box;"\
-"  padding-left: 24px;"\
-"  padding-right: 24px;"\
-"  padding-top: 3px;"\
-"  padding-bottom: 4px;"\
-"  border-radius: 0;"\
 "}"\
 ".color-%1$d.slanted:backdrop {"\
 "  background-image: linear-gradient(100deg, /* left edge */"\
@@ -157,7 +149,6 @@
 "  background-size: 100%% 100%%, 100%% 100%%;"\
 "  background-origin: padding-box, border-box;"\
 "  background-clip: padding-box, border-box;"\
-"  border-radius: 0 2px 2px 0;"\
 "}"\
 ".color-%1$d.slanted-start:backdrop {"\
 "  background-image: linear-gradient(100deg, /* left edge */"\
@@ -185,7 +176,6 @@
 "                                    @event_shadow_color	17px);"\
 "  border-bottom: 1px solid transparent; /* for the shadow displacement */"\
 "  background-position: right bottom, right bottom;"\
-"  border-radius: 2px 0 0 2px;"\
 "}"\
 ".color-%1$d.slanted-start:dir(rtl):backdrop {"\
 "  background-image: linear-gradient(260deg, /* right edge */"\
@@ -217,7 +207,6 @@
 "  background-size: 100%% 100%%, 100%% 100%%;"\
 "  background-origin: padding-box, border-box;"\
 "  background-clip: padding-box, border-box;"\
-"  border-radius: 2px 0 0 2px;"\
 "}"\
 ".color-%1$d.slanted-end:backdrop {"\
 "  background-image: linear-gradient(280deg, /* right edge */"\
@@ -244,7 +233,6 @@
 "                                    transparent         16px,"\
 "                                    @event_shadow_color	17px);"\
 "  background-position: left bottom, left bottom;"\
-"  border-radius: 0 2px 2px 0;"\
 "}"\
 ".color-%1$d.slanted-end:dir(rtl):backdrop {"\
 "  background-image: linear-gradient(80deg, /* left edge */"\
@@ -254,12 +242,7 @@
 "                                    alpha(%2$s, 0) 10px,"\
 "                                    alpha(%2$s, 0) 16px,"\
 "                                    %2$s	   17px);"\
-"}"\
-".color-%1$d.slanted > stack { margin: 0 16px 0 16px; }"\
-".color-%1$d.slanted-start > stack { margin-left: 16px; }"\
-".color-%1$d.slanted-start:dir(rtl) > stack { margin-right: 16px; }"\
-".color-%1$d.slanted-end > stack { margin-right: 16px; }"\
-".color-%1$d.slanted-end:dir(rtl) > stack { margin-left: 16px; }"
+"}"
 
 #endif /* CSS_CODE_H */
 
