@@ -24,25 +24,36 @@
 #include "gcal-date-chooser.h"
 #include "gcal-date-chooser-day.h"
 #include "gcal-multi-choice.h"
+#include "gcal-range-tree.h"
+#include "gcal-timeline-subscriber.h"
+#include "gcal-view.h"
 
 #include <stdlib.h>
 #include <langinfo.h>
 
+#define ROWS 6
+#define COLS 7
+#define DAYS 42
+
 struct _GcalDateChooser
 {
-  GtkBin              parent;
+  AdwBin              parent;
 
   GtkWidget          *month_choice;
+  GtkWidget          *popover_month_choice;
   GtkWidget          *year_choice;
+  GtkWidget          *combined_choice;
   GtkWidget          *grid;
 
   GtkWidget          *day_grid;
   GtkWidget          *corner;
-  GtkWidget          *cols[7];
-  GtkWidget          *rows[6];
-  GtkWidget          *days[6][7];
+  GtkWidget          *cols[COLS];
+  GtkWidget          *rows[ROWS];
+  GtkWidget          *days[ROWS][COLS];
+  GtkWidget          *week[ROWS];
 
   GDateTime          *date;
+  GcalContext        *context;
 
   gint                this_year;
   gint                week_start;
@@ -50,18 +61,30 @@ struct _GcalDateChooser
   gboolean            show_heading;
   gboolean            show_day_names;
   gboolean            show_week_numbers;
-  gboolean            no_month_change;
+  gboolean            show_selected_week;
+  gboolean            show_events;
+  gboolean            split_month_year;
 
-  GcalDateChooserDayOptionsCallback day_options_cb;
-  gpointer            day_options_data;
-  GDestroyNotify      day_options_destroy;
+  GcalRangeTree      *events;
+
+  gulong              update_indicators_idle_id;
 };
 
-G_DEFINE_TYPE (GcalDateChooser, gcal_date_chooser, GTK_TYPE_BIN)
+static void          gcal_view_interface_init                    (GcalViewInterface  *iface);
+
+static void          gcal_timeline_subscriber_interface_init     (GcalTimelineSubscriberInterface *iface);
+static gboolean      update_event_indicators_in_idle_cb          (gpointer           data);
+
+static void          gcal_date_chooser_set_date                  (GcalView  *view,
+                                                                  GDateTime *date);
+
+G_DEFINE_TYPE_WITH_CODE (GcalDateChooser, gcal_date_chooser, ADW_TYPE_BIN,
+                         G_IMPLEMENT_INTERFACE (GCAL_TYPE_VIEW, gcal_view_interface_init)
+                         G_IMPLEMENT_INTERFACE (GCAL_TYPE_TIMELINE_SUBSCRIBER,
+                                                gcal_timeline_subscriber_interface_init));
 
 enum
 {
-  MONTH_CHANGED,
   DAY_SELECTED,
   LAST_SIGNAL
 };
@@ -69,12 +92,15 @@ enum
 enum
 {
   PROP_0,
-  PROP_DATE,
   PROP_SHOW_HEADING,
   PROP_SHOW_DAY_NAMES,
   PROP_SHOW_WEEK_NUMBERS,
-  PROP_NO_MONTH_CHANGE,
-  NUM_PROPERTIES
+  PROP_SHOW_SELECTED_WEEK,
+  PROP_SHOW_EVENTS,
+  PROP_SPLIT_MONTH_YEAR,
+  PROP_DATE,
+  PROP_CONTEXT,
+  NUM_PROPERTIES = PROP_SPLIT_MONTH_YEAR + 1,
 };
 
 static guint signals[LAST_SIGNAL] = { 0, };
@@ -86,6 +112,22 @@ static const guint month_length[2][13] =
   { 0, 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 }
 };
 
+static gboolean
+get_combined_choice_visible (gpointer user_data,
+                             gboolean show_heading,
+                             gboolean split_month_year)
+{
+  return show_heading && !split_month_year;
+}
+
+static gboolean
+get_split_choice_visible (gpointer user_data,
+                          gboolean show_heading,
+                          gboolean split_month_year)
+{
+  return show_heading && split_month_year;
+}
+
 static gboolean
 leap (guint year)
 {
@@ -113,9 +155,9 @@ calendar_compute_days (GcalDateChooser *self)
   first_day = g_date_time_get_day_of_week (date);
   g_date_time_unref (date);
 
-  first_day = (first_day + 7 - self->week_start) % 7;
+  first_day = (first_day + COLS - self->week_start) % COLS;
   if (first_day == 0)
-    first_day = 7;
+    first_day = COLS;
 
   /* Compute days of previous month */
   if (month > 1)
@@ -143,8 +185,8 @@ calendar_compute_days (GcalDateChooser *self)
     }
 
   /* Compute days of current month */
-  row = first_day / 7;
-  col = first_day % 7;
+  row = first_day / COLS;
+  col = first_day % COLS;
 
   for (day = 1; day <= ndays_in_month; day++)
     {
@@ -155,7 +197,7 @@ calendar_compute_days (GcalDateChooser *self)
       g_date_time_unref (date);
 
       col++;
-      if (col == 7)
+      if (col == COLS)
         {
           row++;
           col = 0;
@@ -173,9 +215,9 @@ calendar_compute_days (GcalDateChooser *self)
     }
 
   day = 1;
-  for (; row < 6; row++)
+  for (; row < ROWS; row++)
     {
-      for (; col < 7; col++)
+      for (; col < COLS; col++)
         {
           d = GCAL_DATE_CHOOSER_DAY (self->days[row][col]);
           date = g_date_time_new_local (other_year, other_month, day, 1, 1, 1);
@@ -188,18 +230,16 @@ calendar_compute_days (GcalDateChooser *self)
     }
 
   /* update week numbers */
-  for (row = 0; row < 6; row++)
+  for (row = 0; row < ROWS; row++)
     {
       gchar *text;
 
-      d = GCAL_DATE_CHOOSER_DAY (self->days[row][6]);
+      d = GCAL_DATE_CHOOSER_DAY (self->days[row][COLS - 1]);
       date = gcal_date_chooser_day_get_date (d);
       text = g_strdup_printf ("%d", g_date_time_get_week_of_year (date));
       gtk_label_set_label (GTK_LABEL (self->rows[row]), text);
       g_free (text);
     }
-
-  gcal_date_chooser_invalidate_day_options (self);
 }
 
 /* 0 == sunday */
@@ -230,9 +270,9 @@ calendar_init_weekday_display (GcalDateChooser *self)
   gint i;
   gchar *text;
 
-  for (i = 0; i < 7; i++)
+  for (i = 0; i < COLS; i++)
     {
-      text = calendar_get_weekday_name ((i + self->week_start) % 7);
+      text = calendar_get_weekday_name ((i + self->week_start) % COLS);
       gtk_label_set_label (GTK_LABEL (self->cols[i]), text);
       g_free (text);
     }
@@ -246,6 +286,24 @@ format_month (GcalMultiChoice *choice,
   return g_strdup (gcal_get_month_name (value));
 }
 
+static gchar *
+format_month_year (GcalMultiChoice *choice,
+                   gint             value,
+                   gpointer         data)
+{
+  GcalDateChooser *self = GCAL_DATE_CHOOSER (data);
+  g_autoptr (GDateTime) now = g_date_time_new_now (g_date_time_get_timezone (self->date));
+  gint year, month;
+
+  year = value / 12;
+  month = value % 12;
+
+  if (g_date_time_get_year (now) == year)
+    return g_strdup (gcal_get_month_name (month));
+  else
+    return g_strdup_printf ("%s %d", gcal_get_month_name (month), year);
+}
+
 static void
 calendar_init_month_display (GcalDateChooser *self)
 {
@@ -263,6 +321,8 @@ calendar_init_month_display (GcalDateChooser *self)
 
   gcal_multi_choice_set_choices (GCAL_MULTI_CHOICE (self->month_choice),
                                 (const gchar**) months);
+  gcal_multi_choice_set_choices (GCAL_MULTI_CHOICE (self->popover_month_choice),
+                                (const gchar**) months);
 
   for (i = 0; i < 12; i++)
     g_free (months[i]);
@@ -271,6 +331,15 @@ calendar_init_month_display (GcalDateChooser *self)
                                          format_month,
                                          self,
                                          NULL);
+  gcal_multi_choice_set_format_callback (GCAL_MULTI_CHOICE (self->popover_month_choice),
+                                         format_month,
+                                         self,
+                                         NULL);
+
+  gcal_multi_choice_set_format_callback (GCAL_MULTI_CHOICE (self->combined_choice),
+                                         format_month_year,
+                                         self,
+                                         NULL);
 }
 
 static void
@@ -280,14 +349,22 @@ calendar_update_selected_day_display (GcalDateChooser *self)
   GDateTime *date;
   gint row, col;
 
-  for (row = 0; row < 6; row++)
+  for (row = 0; row < ROWS; row++)
     {
-      for (col = 0; col < 7; col++)
+      gboolean row_selected = FALSE;
+
+      for (col = 0; col < COLS; col++)
       {
+        gboolean day_selected;
+
         d = GCAL_DATE_CHOOSER_DAY (self->days[row][col]);
         date = gcal_date_chooser_day_get_date (d);
-        gcal_date_chooser_day_set_selected (d, gcal_date_time_compare_date (date, self->date) == 0);
+        day_selected = gcal_date_time_compare_date (date, self->date) == 0;
+        gcal_date_chooser_day_set_selected (d, day_selected);
+        row_selected |= day_selected;
       }
+
+      gtk_widget_set_visible (self->week[row], row_selected && self->show_selected_week);
     }
 }
 
@@ -305,7 +382,7 @@ calendar_update_selected_day (GcalDateChooser *self)
   if (month_len < day)
     {
       date = g_date_time_new_local (year, month, month_len, 1, 1, 1);
-      gcal_date_chooser_set_date (self, date);
+      gcal_date_chooser_set_date (GCAL_VIEW (self), date);
       g_date_time_unref (date);
     }
   else
@@ -314,11 +391,208 @@ calendar_update_selected_day (GcalDateChooser *self)
     }
 }
 
+static void
+update_event_indicators (GcalDateChooser *self)
+{
+  gint row, col;
+
+  for (row = 0; row < ROWS; row++)
+    {
+      for (col = 0; col < COLS; col++)
+      {
+          GDateTime *date;
+          g_autoptr (GcalRange) range = NULL;
+
+          date = gcal_date_chooser_day_get_date (GCAL_DATE_CHOOSER_DAY (self->days[row][col]));
+          range = gcal_range_new_take (g_date_time_ref (date),
+                                       g_date_time_add_days (date, 1),
+                                       GCAL_RANGE_DEFAULT);
+
+          gcal_date_chooser_day_set_dot_visible (GCAL_DATE_CHOOSER_DAY (self->days[row][col]),
+                                                 gcal_range_tree_count_entries_at_range (self->events, range) > 0);
+      }
+    }
+}
+
+static void
+queue_update_event_indicators (GcalDateChooser *self)
+{
+  if (self->update_indicators_idle_id > 0)
+    return;
+
+  self->update_indicators_idle_id = g_idle_add_full (G_PRIORITY_LOW,
+                                                     update_event_indicators_in_idle_cb,
+                                                     self,
+                                                     NULL);
+}
+
 static void
 day_selected_cb (GcalDateChooserDay *d,
                  GcalDateChooser    *self)
 {
-  gcal_date_chooser_set_date (self, gcal_date_chooser_day_get_date (d));
+  gcal_date_chooser_set_date (GCAL_VIEW (self), gcal_date_chooser_day_get_date (d));
+}
+
+static void
+on_clock_day_changed_cb (GcalDateChooser *self)
+{
+  /* FIXME Update the widget to the new day. */
+}
+
+static gboolean
+update_event_indicators_in_idle_cb (gpointer data)
+{
+  GcalDateChooser *self = GCAL_DATE_CHOOSER (data);
+
+  update_event_indicators (self);
+  self->update_indicators_idle_id = 0;
+
+  return G_SOURCE_REMOVE;
+}
+
+/*
+ * GcalView interface
+ */
+
+static void
+gcal_date_chooser_set_date (GcalView  *view,
+                            GDateTime *date)
+{
+  GcalDateChooser *self = GCAL_DATE_CHOOSER (view);
+  gint y1, m1, d1, y2, m2, d2;
+
+  g_object_freeze_notify (G_OBJECT (self));
+
+  g_date_time_get_ymd (self->date, &y1, &m1, &d1);
+  g_date_time_get_ymd (date, &y2, &m2, &d2);
+
+  gcal_set_date_time (&self->date, date);
+
+  if (y1 != y2 || m1 != m2)
+    {
+      gcal_multi_choice_set_value (GCAL_MULTI_CHOICE (self->year_choice), y2);
+      gcal_multi_choice_set_value (GCAL_MULTI_CHOICE (self->month_choice), m2 - 1);
+      gcal_multi_choice_set_value (GCAL_MULTI_CHOICE (self->combined_choice), y2 * 12 + m2 - 1);
+      calendar_compute_days (self);
+      gcal_timeline_subscriber_range_changed (GCAL_TIMELINE_SUBSCRIBER (self));
+    }
+
+  if (y1 != y2 || m1 != m2 || d1 != d2)
+    {
+      calendar_update_selected_day (self);
+      g_signal_emit (self, signals[DAY_SELECTED], 0);
+      g_object_notify (G_OBJECT (self), "active-date");
+    }
+
+  g_object_thaw_notify (G_OBJECT (self));
+}
+
+static GDateTime*
+gcal_date_chooser_get_date (GcalView *view)
+{
+  GcalDateChooser *self = GCAL_DATE_CHOOSER (view);
+
+  return self->date;
+}
+
+static GList*
+gcal_date_chooser_get_children_by_uuid (GcalView              *view,
+                                        GcalRecurrenceModType  mod,
+                                        const gchar           *uuid)
+{
+  return NULL;
+}
+
+static GDateTime*
+gcal_date_chooser_get_next_date (GcalView *view)
+{
+  GcalDateChooser *self = GCAL_DATE_CHOOSER (view);
+
+  g_assert (self->date != NULL);
+  return g_date_time_add_months (self->date, 1);
+}
+
+static GDateTime*
+gcal_date_chooser_get_previous_date (GcalView *view)
+{
+  GcalDateChooser *self = GCAL_DATE_CHOOSER (view);
+
+  g_assert (self->date != NULL);
+  return g_date_time_add_months (self->date, -1);
+}
+
+static void
+gcal_view_interface_init (GcalViewInterface *iface)
+{
+  iface->get_date = gcal_date_chooser_get_date;
+  iface->set_date = gcal_date_chooser_set_date;
+  iface->get_children_by_uuid = gcal_date_chooser_get_children_by_uuid;
+  iface->get_next_date = gcal_date_chooser_get_next_date;
+  iface->get_previous_date = gcal_date_chooser_get_previous_date;
+}
+
+
+/*
+ * GcalTimelineSubscriber implementation
+ */
+
+static GcalRange*
+gcal_date_chooser_get_range (GcalTimelineSubscriber *subscriber)
+{
+  GcalDateChooser *self;
+  GDateTime *start;
+  g_autoptr (GDateTime) end = NULL;
+
+  self = GCAL_DATE_CHOOSER (subscriber);
+  start = gcal_date_chooser_day_get_date (GCAL_DATE_CHOOSER_DAY (self->days[0][0]));
+  end = g_date_time_add_days (start, G_N_ELEMENTS (self->days) * G_N_ELEMENTS (self->days[0]));
+
+  return gcal_range_new (start, end, GCAL_RANGE_DEFAULT);
+}
+
+static void
+gcal_date_chooser_add_event (GcalTimelineSubscriber *subscriber,
+                             GcalEvent              *event)
+{
+  GcalDateChooser *self;
+
+  self = GCAL_DATE_CHOOSER (subscriber);
+
+  g_debug ("Caching event '%s' in %s", gcal_event_get_uid (event), G_OBJECT_TYPE_NAME (self));
+  gcal_range_tree_add_range (self->events, gcal_event_get_range (event), g_object_ref (event));
+
+  queue_update_event_indicators (self);
+}
+
+static void
+gcal_date_chooser_remove_event (GcalTimelineSubscriber *subscriber,
+                                GcalEvent              *event)
+{
+  GcalDateChooser *self;
+
+  self = GCAL_DATE_CHOOSER (subscriber);
+
+  g_debug ("Removing event '%s' from %s's cache", gcal_event_get_uid (event), G_OBJECT_TYPE_NAME (self));
+  gcal_range_tree_remove_range (self->events, gcal_event_get_range (event), event);
+
+  queue_update_event_indicators (self);
+}
+
+static void
+gcal_date_chooser_update_event (GcalTimelineSubscriber *subscriber,
+                                GcalEvent              *event)
+{
+  gcal_date_chooser_remove_event (subscriber, event);
+  gcal_date_chooser_add_event (subscriber, event);
+}
+
+static void
+gcal_timeline_subscriber_interface_init (GcalTimelineSubscriberInterface *iface)
+{
+  iface->get_range = gcal_date_chooser_get_range;
+  iface->add_event = gcal_date_chooser_add_event;
+  iface->remove_event = gcal_date_chooser_remove_event;
+  iface->update_event = gcal_date_chooser_update_event;
 }
 
 static void
@@ -332,7 +606,18 @@ calendar_set_property (GObject      *obj,
   switch (property_id)
     {
     case PROP_DATE:
-      gcal_date_chooser_set_date (self, g_value_get_boxed (value));
+      gcal_date_chooser_set_date (GCAL_VIEW (self), g_value_get_boxed (value));
+      break;
+
+    case PROP_CONTEXT:
+      g_assert (self->context == NULL);
+      self->context = g_value_dup_object (value);
+
+      g_signal_connect_object (gcal_context_get_clock (self->context),
+                               "day-changed",
+                               G_CALLBACK (on_clock_day_changed_cb),
+                               self,
+                               G_CONNECT_SWAPPED);
       break;
 
     case PROP_SHOW_HEADING:
@@ -347,8 +632,16 @@ calendar_set_property (GObject      *obj,
       gcal_date_chooser_set_show_week_numbers (self, g_value_get_boolean (value));
       break;
 
-    case PROP_NO_MONTH_CHANGE:
-      gcal_date_chooser_set_no_month_change (self, g_value_get_boolean (value));
+    case PROP_SHOW_SELECTED_WEEK:
+      gcal_date_chooser_set_show_selected_week (self, g_value_get_boolean (value));
+      break;
+
+    case PROP_SHOW_EVENTS:
+      gcal_date_chooser_set_show_events (self, g_value_get_boolean (value));
+      break;
+
+    case PROP_SPLIT_MONTH_YEAR:
+      gcal_date_chooser_set_split_month_year (self, g_value_get_boolean (value));
       break;
 
     default:
@@ -371,6 +664,10 @@ calendar_get_property (GObject    *obj,
       g_value_set_boxed (value, self->date);
       break;
 
+    case PROP_CONTEXT:
+      g_value_set_object (value, self->context);
+      break;
+
     case PROP_SHOW_HEADING:
       g_value_set_boolean (value, self->show_heading);
       break;
@@ -383,8 +680,16 @@ calendar_get_property (GObject    *obj,
       g_value_set_boolean (value, self->show_week_numbers);
       break;
 
-    case PROP_NO_MONTH_CHANGE:
-      g_value_set_boolean (value, self->no_month_change);
+    case PROP_SHOW_SELECTED_WEEK:
+      g_value_set_boolean (value, self->show_selected_week);
+      break;
+
+    case PROP_SHOW_EVENTS:
+      g_value_set_boolean (value, self->show_events);
+      break;
+
+    case PROP_SPLIT_MONTH_YEAR:
+      g_value_set_boolean (value, self->split_month_year);
       break;
 
     default:
@@ -394,65 +699,76 @@ calendar_get_property (GObject    *obj,
 }
 
 static void
-multi_choice_changed (GcalDateChooser *self)
+multi_choice_changed (GcalDateChooser *self,
+                      gint             year,
+                      gint             month)
 {
   g_autoptr (GDateTime) date = NULL;
-  gint year, month, day;
+  gint day;
 
-  year = gcal_multi_choice_get_value (GCAL_MULTI_CHOICE (self->year_choice));
-  month = gcal_multi_choice_get_value (GCAL_MULTI_CHOICE (self->month_choice)) + 1;
   day = g_date_time_get_day_of_month (self->date);
 
   /* Make sure the day is valid at that month */
   day = MIN (day, month_length[leap (year)][month]);
 
   date = g_date_time_new_local (year, month, day, 1, 1, 1);
-  gcal_date_chooser_set_date (self, date);
+  gcal_date_chooser_set_date (GCAL_VIEW (self), date);
 }
 
 static void
-calendar_drag_data_received (GtkWidget        *widget,
-                             GdkDragContext   *context,
-                             gint              x,
-                             gint              y,
-                             GtkSelectionData *selection_data,
-                             guint             info,
-                             guint             time)
+combined_multi_choice_changed (GcalDateChooser *self)
 {
-  GcalDateChooser *self = GCAL_DATE_CHOOSER (widget);
-  gchar *text;
-  GDate *gdate;
+  gint year, month;
+
+  year = gcal_multi_choice_get_value (GCAL_MULTI_CHOICE (self->combined_choice)) / 12;
+  month = gcal_multi_choice_get_value (GCAL_MULTI_CHOICE (self->combined_choice)) % 12 + 1;
+
+  multi_choice_changed (self, year, month);
+}
+
+static void
+split_multi_choice_changed (GcalDateChooser *self)
+{
+  gint year, month;
+
+  year = gcal_multi_choice_get_value (GCAL_MULTI_CHOICE (self->year_choice));
+  month = gcal_multi_choice_get_value (GCAL_MULTI_CHOICE (self->month_choice)) + 1;
+
+  multi_choice_changed (self, year, month);
+}
+
+static gboolean
+on_drop_target_drop_cb (GtkDropTarget   *target,
+                        const GValue    *value,
+                        double           x,
+                        double           y,
+                        GcalDateChooser *self)
+{
+  g_autoptr (GDateTime) date = NULL;
+  g_autoptr (GDate) gdate = NULL;
+  const gchar *text;
   gint year, month, day;
-  GDateTime *date;
 
   gdate = g_date_new ();
-  text = (gchar *)gtk_selection_data_get_text (selection_data);
+  text = g_value_get_string (value);
+
   if (text)
-    {
-      g_date_set_parse (gdate, text);
-      g_free (text);
-    }
+    g_date_set_parse (gdate, text);
+
   if (!g_date_valid (gdate))
-    {
-      g_date_free (gdate);
-      gtk_drag_finish (context, FALSE, FALSE, time);
-      return;
-    }
+    return FALSE;
 
   year = g_date_get_year (gdate);
   month = g_date_get_month (gdate);
   day = g_date_get_day (gdate);
 
-  g_date_free (gdate);
-
-  gtk_drag_finish (context, TRUE, FALSE, time);
-
-  if (!self->show_heading || self->no_month_change)
+  if (!self->show_heading)
     g_date_time_get_ymd (self->date, &year, &month, NULL);
 
   date = g_date_time_new_local (year, month, day, 1, 1, 1);
-  gcal_date_chooser_set_date (self, date);
-  g_date_time_unref (date);
+  gcal_date_chooser_set_date (GCAL_VIEW (self), date);
+
+  return TRUE;
 }
 
 static void
@@ -460,7 +776,10 @@ gcal_date_chooser_finalize (GObject *object)
 {
   GcalDateChooser *self = GCAL_DATE_CHOOSER (object);
 
+  g_clear_object (&self->context);
   g_clear_pointer (&self->date, g_date_time_unref);
+  g_clear_pointer (&self->events, gcal_range_tree_unref);
+
   G_OBJECT_CLASS (gcal_date_chooser_parent_class)->finalize (object);
 }
 
@@ -483,16 +802,6 @@ gcal_date_chooser_class_init (GcalDateChooserClass *class)
   object_class->set_property = calendar_set_property;
   object_class->get_property = calendar_get_property;
 
-  widget_class->drag_data_received = calendar_drag_data_received;
-
-  gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/calendar/ui/event-editor/gcal-date-chooser.ui");
-
-  properties[PROP_DATE] = g_param_spec_boxed ("date",
-                                              "Date",
-                                              "The selected date",
-                                              G_TYPE_DATE_TIME,
-                                              G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
-
   properties[PROP_SHOW_HEADING] = g_param_spec_boolean ("show-heading",
                                                         "Show Heading",
                                                         "If TRUE, a heading is displayed",
@@ -511,21 +820,33 @@ gcal_date_chooser_class_init (GcalDateChooserClass *class)
                                                              TRUE,
                                                              G_PARAM_READWRITE);
 
-  properties[PROP_NO_MONTH_CHANGE] = g_param_spec_boolean ("no-month-change",
-                                                           "No Month Change",
-                                                           "If TRUE, the selected month cannot be changed",
-                                                           FALSE,
-                                                           G_PARAM_READWRITE);
+  /**
+   * GcalDateChooser:show-selected-week:
+   *
+   * Whether the selected week is highlighted or not.
+   */
+  properties[PROP_SHOW_SELECTED_WEEK] = g_param_spec_boolean ("show-selected-week",
+                                                              "Show Selected Week",
+                                                              "If TRUE, week numbers are displayed",
+                                                              TRUE,
+                                                              G_PARAM_READWRITE);
+
+  properties[PROP_SHOW_EVENTS] = g_param_spec_boolean ("show-events",
+                                                       "Show Events",
+                                                       "If TRUE, events are displayed",
+                                                       TRUE,
+                                                       G_PARAM_READWRITE);
+
+  properties[PROP_SPLIT_MONTH_YEAR] = g_param_spec_boolean ("split-month-year",
+                                                            "Split Month Year",
+                                                            "If TRUE, the month and year selectors are split",
+                                                            TRUE,
+                                                            G_PARAM_READWRITE);
 
   g_object_class_install_properties (object_class, NUM_PROPERTIES, properties);
 
-  signals[MONTH_CHANGED] = g_signal_new ("month-changed",
-                                         G_OBJECT_CLASS_TYPE (object_class),
-                                         G_SIGNAL_RUN_FIRST,
-                                         0,
-                                         NULL, NULL,
-                                         NULL,
-                                         G_TYPE_NONE, 0);
+  g_object_class_override_property (object_class, PROP_DATE, "active-date");
+  g_object_class_override_property (object_class, PROP_CONTEXT, "context");
 
   signals[DAY_SELECTED] = g_signal_new ("day-selected",
                                         G_OBJECT_CLASS_TYPE (object_class),
@@ -535,36 +856,73 @@ gcal_date_chooser_class_init (GcalDateChooserClass *class)
                                         NULL,
                                         G_TYPE_NONE, 0);
 
+  gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/calendar/ui/event-editor/gcal-date-chooser.ui");
+
+  gtk_widget_class_bind_template_child (widget_class, GcalDateChooser, combined_choice);
   gtk_widget_class_bind_template_child (widget_class, GcalDateChooser, month_choice);
+  gtk_widget_class_bind_template_child (widget_class, GcalDateChooser, popover_month_choice);
   gtk_widget_class_bind_template_child (widget_class, GcalDateChooser, year_choice);
   gtk_widget_class_bind_template_child (widget_class, GcalDateChooser, grid);
 
-  gtk_widget_class_bind_template_callback (widget_class, multi_choice_changed);
+  gtk_widget_class_bind_template_callback (widget_class, combined_multi_choice_changed);
+  gtk_widget_class_bind_template_callback (widget_class, get_combined_choice_visible);
+  gtk_widget_class_bind_template_callback (widget_class, get_split_choice_visible);
+  gtk_widget_class_bind_template_callback (widget_class, split_multi_choice_changed);
 
   gtk_widget_class_set_css_name (widget_class, "datechooser");
 }
 
 
+static gboolean
+show_week_number_to_column_cb (GBinding     *binding,
+                               const GValue *from_value,
+                               GValue       *to_value,
+                               gpointer      user_data)
+{
+  g_value_set_int (to_value, g_value_get_boolean (from_value) ? -1 : 0);
+
+  return TRUE;
+}
+
+
+static gboolean
+show_week_number_to_column_span_cb (GBinding     *binding,
+                                    const GValue *from_value,
+                                    GValue       *to_value,
+                                    gpointer      user_data)
+{
+  g_value_set_int (to_value, g_value_get_boolean (from_value) ? COLS + 1 : COLS);
+
+  return TRUE;
+}
+
+
 static void
 gcal_date_chooser_init (GcalDateChooser *self)
 {
+  GtkDropTarget *drop_target;
   GtkWidget *label;
   gint row, col;
   gint year, month;
+  GtkLayoutManager *layout_manager;
 
   self->show_heading = TRUE;
   self->show_day_names = TRUE;
   self->show_week_numbers = TRUE;
-  self->no_month_change = FALSE;
+  self->show_selected_week = TRUE;
+  self->show_events = TRUE;
+  self->split_month_year = TRUE;
 
   self->date = g_date_time_new_now_local ();
   g_date_time_get_ymd (self->date, &self->this_year, NULL, NULL);
 
   self->week_start = get_first_weekday ();
 
+  self->events = gcal_range_tree_new_with_free_func (g_object_unref);
+
   gtk_widget_init_template (GTK_WIDGET (self));
 
-  for (col = 0; col < 7; col++)
+  for (col = 0; col < COLS; col++)
     {
       self->cols[col] = gtk_label_new ("");
 
@@ -574,13 +932,17 @@ gcal_date_chooser_init (GcalDateChooser *self)
                               "visible",
                               G_BINDING_SYNC_CREATE);
 
-      gtk_style_context_add_class (gtk_widget_get_style_context (self->cols[col]), "weekday");
+      gtk_widget_add_css_class (self->cols[col], "weekday");
 
       gtk_grid_attach (GTK_GRID (self->grid), self->cols[col], col, -1, 1, 1);
     }
 
-  for (row = 0; row < 6; row++)
+  layout_manager = gtk_widget_get_layout_manager (self->grid);
+
+  for (row = 0; row < ROWS; row++)
     {
+      GtkLayoutChild *layout_child;
+
       self->rows[row] = gtk_label_new ("");
 
       g_object_bind_property (self,
@@ -589,9 +951,35 @@ gcal_date_chooser_init (GcalDateChooser *self)
                               "visible",
                               G_BINDING_SYNC_CREATE);
 
-      gtk_widget_show (self->rows[row]);
-      gtk_style_context_add_class (gtk_widget_get_style_context (self->rows[row]), "weeknum");
+      gtk_widget_add_css_class (self->rows[row], "weeknum");
       gtk_grid_attach (GTK_GRID (self->grid), self->rows[row], -1, row, 1, 1);
+
+      self->week[row] = adw_bin_new ();
+      gtk_widget_set_visible (self->week[row], FALSE);
+      gtk_widget_add_css_class (self->week[row], "current-week");
+      gtk_grid_attach (GTK_GRID (self->grid), self->week[row], -1, row, 8, 1);
+
+      layout_child = gtk_layout_manager_get_layout_child (layout_manager, self->week[row]);
+
+      g_object_bind_property_full (self,
+                                   "show-week-numbers",
+                                   layout_child,
+                                   "column",
+                                   G_BINDING_SYNC_CREATE,
+                                   show_week_number_to_column_cb,
+                                   NULL,
+                                   NULL,
+                                   NULL);
+
+      g_object_bind_property_full (self,
+                                   "show-week-numbers",
+                                   layout_child,
+                                   "column-span",
+                                   G_BINDING_SYNC_CREATE,
+                                   show_week_number_to_column_span_cb,
+                                   NULL,
+                                   NULL,
+                                   NULL);
     }
 
   /* We are using a stack here to keep the week number column from shrinking
@@ -599,14 +987,14 @@ gcal_date_chooser_init (GcalDateChooser *self)
    */
   self->corner = gtk_stack_new ();
   gtk_grid_attach (GTK_GRID (self->grid), self->corner, -1, -1, 1, 1);
+
   label = gtk_label_new ("");
-  gtk_widget_show (label);
-  gtk_style_context_add_class (gtk_widget_get_style_context (label), "weekday");
-  gtk_container_add (GTK_CONTAINER (self->corner), label);
+  gtk_widget_add_css_class (label, "weekday");
+  gtk_stack_add_named (GTK_STACK (self->corner), label, "weekday");
 
   label = gtk_label_new ("99");
-  gtk_style_context_add_class (gtk_widget_get_style_context (label), "weeknum");
-  gtk_container_add (GTK_CONTAINER (self->corner), label);
+  gtk_widget_add_css_class (label, "weeknum");
+  gtk_stack_add_named (GTK_STACK (self->corner), label, "weeknum");
 
   self->day_grid = g_object_new (GTK_TYPE_GRID,
                                  "valign", GTK_ALIGN_FILL,
@@ -615,21 +1003,26 @@ gcal_date_chooser_init (GcalDateChooser *self)
                                  "column-spacing", 2,
                                  "visible", TRUE,
                                  NULL);
-  gtk_grid_attach (GTK_GRID (self->grid), self->day_grid, 0, 0, 7, 6);
+  gtk_grid_attach (GTK_GRID (self->grid), self->day_grid, 0, 0, COLS, ROWS);
 
-  for (row = 0; row < 6; row++)
+  for (row = 0; row < ROWS; row++)
     {
-      for (col = 0; col < 7; col++)
+      for (col = 0; col < COLS; col++)
         {
           self->days[row][col] = gcal_date_chooser_day_new ();
 
+          g_object_bind_property (self,
+                                  "show-events",
+                                  self->days[row][col],
+                                  "has-dot",
+                                  G_BINDING_SYNC_CREATE);
+
           g_signal_connect_object (self->days[row][col],
-                                   "selected",
+                                   "clicked",
                                    G_CALLBACK (day_selected_cb),
                                    self,
                                    0);
 
-          gtk_widget_show (self->days[row][col]);
           gtk_grid_attach (GTK_GRID (self->day_grid), self->days[row][col], col, row, 1, 1);
         }
     }
@@ -641,10 +1034,13 @@ gcal_date_chooser_init (GcalDateChooser *self)
   g_date_time_get_ymd (self->date, &year, &month, NULL);
   gcal_multi_choice_set_value (GCAL_MULTI_CHOICE (self->year_choice), year);
   gcal_multi_choice_set_value (GCAL_MULTI_CHOICE (self->month_choice), month - 1);
+  gcal_multi_choice_set_value (GCAL_MULTI_CHOICE (self->combined_choice), year * 12 + month - 1);
   calendar_update_selected_day_display (self);
 
-  gtk_drag_dest_set (GTK_WIDGET (self), GTK_DEST_DEFAULT_ALL, NULL, 0, GDK_ACTION_COPY);
-  gtk_drag_dest_add_text_targets (GTK_WIDGET (self));
+  drop_target = gtk_drop_target_new (G_TYPE_STRING, GDK_ACTION_COPY);
+  gtk_drop_target_set_preload (drop_target, TRUE);
+  g_signal_connect (drop_target, "drop", G_CALLBACK (on_drop_target_drop_cb), NULL);
+  gtk_widget_add_controller (GTK_WIDGET (self), GTK_EVENT_CONTROLLER (drop_target));
 }
 
 GtkWidget*
@@ -711,99 +1107,73 @@ gcal_date_chooser_get_show_week_numbers (GcalDateChooser *self)
   return self->show_week_numbers;
 }
 
+/**
+ * gcal_date_chooser_set_show_selected_week:
+ * @self: a #GcalDateChooser
+ * @setting: whether the selected week is highlighted or not.
+ *
+ * Sets the selected week is highlighted or not.
+ */
 void
-gcal_date_chooser_set_no_month_change (GcalDateChooser *self,
-                                       gboolean         setting)
+gcal_date_chooser_set_show_selected_week (GcalDateChooser *self,
+                                          gboolean         setting)
 {
-  if (self->no_month_change == setting)
+  if (self->show_selected_week == setting)
     return;
 
-  self->no_month_change = setting;
+  self->show_selected_week = setting;
 
-  g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_NO_MONTH_CHANGE]);
+  calendar_update_selected_day_display (self);
+
+  g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_SHOW_SELECTED_WEEK]);
+}
+
+/**
+ * gcal_date_chooser_get_show_selected_week:
+ * @self: a #GcalDateChooser
+ *
+ * Returns: whether the selected week is highlighted or not.
+ */
+gboolean
+gcal_date_chooser_get_show_selected_week (GcalDateChooser *self)
+{
+  return self->show_selected_week;
+}
+
+void
+gcal_date_chooser_set_show_events (GcalDateChooser *self,
+                                   gboolean         setting)
+{
+  if (self->show_events == setting)
+    return;
+
+  self->show_events = setting;
+
+  calendar_update_selected_day_display (self);
+
+  g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_SHOW_EVENTS]);
 }
 
 gboolean
-gcal_date_chooser_get_no_month_change (GcalDateChooser *self)
+gcal_date_chooser_get_show_events (GcalDateChooser *self)
 {
-  return self->no_month_change;
+  return self->show_events;
 }
 
 void
-gcal_date_chooser_set_date (GcalDateChooser *self,
-                            GDateTime       *date)
+gcal_date_chooser_set_split_month_year (GcalDateChooser *self,
+                                        gboolean         setting)
 {
-  gint y1, m1, d1, y2, m2, d2;
+  if (self->split_month_year == setting)
+    return;
 
-  g_object_freeze_notify (G_OBJECT (self));
+  self->split_month_year = setting;
 
-  g_date_time_get_ymd (self->date, &y1, &m1, &d1);
-  g_date_time_get_ymd (date, &y2, &m2, &d2);
-
-  g_date_time_unref (self->date);
-  self->date = g_date_time_ref (date);
-
-  if (y1 != y2 || m1 != m2)
-    {
-      gcal_multi_choice_set_value (GCAL_MULTI_CHOICE (self->year_choice), y2);
-      gcal_multi_choice_set_value (GCAL_MULTI_CHOICE (self->month_choice), m2 - 1);
-      calendar_compute_days (self);
-    }
-
-  if (y1 != y2 || m1 != m2 || d1 != d2)
-    {
-      calendar_update_selected_day (self);
-      g_signal_emit (self, signals[DAY_SELECTED], 0);
-      g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_DATE]);
-    }
-
-  g_object_thaw_notify (G_OBJECT (self));
+  g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_SPLIT_MONTH_YEAR]);
 }
 
-GDateTime *
-gcal_date_chooser_get_date (GcalDateChooser *self)
+gboolean
+gcal_date_chooser_get_split_month_year (GcalDateChooser *self)
 {
-  return self->date;
-}
-
-void
-gcal_date_chooser_set_day_options_callback (GcalDateChooser                   *self,
-                                            GcalDateChooserDayOptionsCallback  callback,
-                                            gpointer                           data,
-                                            GDestroyNotify                     destroy)
-{
-  if (self->day_options_destroy)
-    self->day_options_destroy (self->day_options_data);
-
-  self->day_options_cb = callback;
-  self->day_options_data = data;
-  self->day_options_destroy = destroy;
-
-  gcal_date_chooser_invalidate_day_options (self);
-}
-
-
-void
-gcal_date_chooser_invalidate_day_options (GcalDateChooser *self)
-{
-  GcalDateChooserDayOptions options;
-  GcalDateChooserDay *d;
-  GDateTime *date;
-  gint row, col;
-
-  for (row = 0; row < 6; row++)
-    {
-      for (col = 0; col < 7; col++)
-        {
-          d = GCAL_DATE_CHOOSER_DAY (self->days[row][col]);
-          date = gcal_date_chooser_day_get_date (d);
-
-          if (self->day_options_cb)
-            options = self->day_options_cb (self, date, self->day_options_data);
-          else
-            options = GCAL_DATE_CHOOSER_DAY_NONE;
-
-          gcal_date_chooser_day_set_options (d, options);
-        }
-    }
+  return self->split_month_year;
 }
