@@ -182,6 +182,7 @@ gcal_application_show_about (GSimpleAction *simple,
                         "copyright", copyright,
                         "website", "https://wiki.gnome.org/Apps/Calendar",
                         "issue-url", "https://gitlab.gnome.org/GNOME/gnome-calendar/-/issues/new",
+                        "support-url", "https://discourse.gnome.org/tag/calendar",
                         "license-type", GTK_LICENSE_GPL_3_0,
                         "developers", developers,
                         "designers", designers,
@@ -278,7 +279,7 @@ gcal_application_activate (GApplication *application)
                                     NULL);
 
       g_object_add_weak_pointer (G_OBJECT (self->window), (gpointer*) &self->window);
-      gtk_widget_show (self->window);
+      gtk_widget_set_visible (self->window, TRUE);
     }
 
   gtk_window_present (GTK_WINDOW (self->window));
