@@ -424,20 +424,20 @@ add_meeting (GcalEventPopover *self,
   g_signal_connect (row, "join-meeting", G_CALLBACK (on_join_meeting_cb), self);
   gtk_list_box_append (self->meetings_listbox, row);
 
-  gtk_widget_show (GTK_WIDGET (self->meetings_listbox));
+  gtk_widget_set_visible (GTK_WIDGET (self->meetings_listbox), TRUE);
 }
 
 static void
 setup_location_label (GcalEventPopover *self)
 {
-  g_autoptr (SoupURI) soup_uri = NULL;
+  g_autoptr (GUri) guri = NULL;
   g_autofree gchar *location = NULL;
 
   location = g_strdup (gcal_event_get_location (self->event));
   g_strstrip (location);
 
-  soup_uri = soup_uri_new (location);
-  if (soup_uri)
+  guri = g_uri_parse (location, SOUP_HTTP_URI_FLAGS | G_URI_FLAGS_PARSE_RELAXED, NULL);
+  if (guri)
     {
       GString *string;
 
@@ -511,20 +511,40 @@ on_edit_button_clicked_cb (GtkButton        *edit_button,
   gtk_popover_popdown (GTK_POPOVER (self));
 }
 
+static void
+on_uri_launched_cb (GObject      *source_object,
+                    GAsyncResult *result,
+                    gpointer      user_data)
+{
+  g_autoptr (GcalEventPopover) self = user_data;
+  g_autoptr (GError) error = NULL;
+
+  if (!gtk_uri_launcher_launch_finish (GTK_URI_LAUNCHER (source_object), result, &error))
+    {
+      g_warning ("Error launching URI: %s", error->message);
+      return;
+    }
+
+  gtk_popover_popdown (GTK_POPOVER (self));
+}
+
 static void
 on_join_meeting_cb (GcalMeetingRow   *meeting_row,
                     const gchar      *url,
                     GcalEventPopover *self)
 {
+  g_autoptr (GtkUriLauncher) uri_launcher = NULL;
   GtkWindow *window;
 
-  window = GTK_WINDOW (gtk_widget_get_native (GTK_WIDGET (self)));
+  window = GTK_WINDOW (gtk_widget_get_root (GTK_WIDGET (self)));
   g_assert (window != NULL);
 
-  gtk_show_uri (window, url, GDK_CURRENT_TIME);
-
-  /* For some reason, gtk_popover_popdown() crashes when called here */
-  gtk_widget_hide (GTK_WIDGET (self));
+  uri_launcher = gtk_uri_launcher_new (url);
+  gtk_uri_launcher_launch (uri_launcher,
+                           window,
+                           NULL,
+                           on_uri_launched_cb,
+                           g_object_ref (self));
 }
 
 static void
@@ -538,6 +558,28 @@ on_time_format_changed_cb (GcalEventPopover *self)
 }
 
 
+/*
+ * GtkWidget overrides
+ */
+
+static void
+gcal_event_popover_map (GtkWidget *widget)
+{
+  GcalEventPopover *self = (GcalEventPopover *) widget;
+  GtkListBoxRow *first_meeting_row;
+
+  GTK_WIDGET_CLASS (gcal_event_popover_parent_class)->map (widget);
+
+  first_meeting_row = gtk_list_box_get_row_at_index (self->meetings_listbox, 0);
+  g_message ("Meeting row: %p (%s)", first_meeting_row, first_meeting_row ? G_OBJECT_TYPE_NAME (first_meeting_row) : "none");
+
+  if (first_meeting_row)
+    gtk_widget_grab_focus (GTK_WIDGET (first_meeting_row));
+  else
+    gtk_widget_grab_focus (self->edit_button);
+}
+
+
 /*
  * GObject overrides
  */
@@ -616,6 +658,8 @@ gcal_event_popover_class_init (GcalEventPopoverClass *klass)
   object_class->get_property = gcal_event_popover_get_property;
   object_class->set_property = gcal_event_popover_set_property;
 
+  widget_class->map = gcal_event_popover_map;
+
   /**
    * GcalEventPopover::context:
    *
