@@ -32,7 +32,7 @@
 #define GCAL_WEATHER_CHECK_INTERVAL_RENEW_DEFAULT (3 * 60 * 60)  /* seconds */
 #define GCAL_WEATHER_CHECK_INTERVAL_NEW_DEFAULT   (5 * 60)       /* seconds */
 #define GCAL_WEATHER_VALID_TIMESPAN_DEFAULT       (24 * 60 * 60) /* seconds */
-#define GCAL_WEATHER_FORECAST_MAX_DAYS_DEFAULT     3
+#define GCAL_WEATHER_FORECAST_MAX_DAYS_DEFAULT     5
 
 #define DAY_SECONDS (24 * 60 * 60)
 
@@ -665,7 +665,7 @@ static void
 update_gclue_location (GcalWeatherService  *self,
                        GClueLocation       *location)
 {
-  GWeatherLocation *wlocation = NULL; /* owned */
+  g_autoptr (GWeatherLocation) wlocation = NULL; /* owned */
 
   if (location)
     {
@@ -682,8 +682,6 @@ update_gclue_location (GcalWeatherService  *self,
     }
 
   update_location (self, wlocation);
-
-  g_clear_pointer (&wlocation, gweather_location_unref);
 }
 
 
@@ -764,13 +762,14 @@ on_gclue_simple_creation_cb (GClueSimple        *_source,
                                self,
                                0);
     }
-
-  g_signal_connect_object (client,
-                           "notify::active",
-                           G_CALLBACK (on_gclue_client_activity_changed_cb),
-                           self,
-                           0);
-
+  if (client)
+    {
+      g_signal_connect_object (client,
+                               "notify::active",
+                               G_CALLBACK (on_gclue_client_activity_changed_cb),
+                               self,
+                               0);
+    }
   GCAL_EXIT;
 }
 
@@ -1118,7 +1117,7 @@ gcal_weather_service_run (GcalWeatherService *self,
       g_cancellable_reset (self->location_cancellable);
 
       gclue_simple_new (APPLICATION_ID,
-                        GCLUE_ACCURACY_LEVEL_EXACT,
+                        GCLUE_ACCURACY_LEVEL_CITY,
                         self->location_cancellable,
                         (GAsyncReadyCallback) on_gclue_simple_creation_cb,
                         self);
@@ -1143,6 +1142,7 @@ void
 gcal_weather_service_stop (GcalWeatherService *self)
 {
   GCAL_ENTRY;
+  GClueClient *client;
 
   g_return_if_fail (GCAL_IS_WEATHER_SERVICE (self));
 
@@ -1164,11 +1164,14 @@ gcal_weather_service_stop (GcalWeatherService *self)
     }
   else
     {
-      gclue_client_call_stop (gclue_simple_get_client (self->location_service),
-                              self->location_cancellable,
-                              (GAsyncReadyCallback) on_gclue_client_stopped_cb,
-                              self->location_service);
-
+      client = gclue_simple_get_client (self->location_service);
+      if (client)
+        {
+          gclue_client_call_stop (client,
+                                  self->location_cancellable,
+                                  (GAsyncReadyCallback) on_gclue_client_stopped_cb,
+                                  self->location_service);
+        }
       self->location_service = NULL;
     }
 
